@@ -1,5 +1,5 @@
 /*
- *  $Id: platform.h,v 1.1 2008-05-30 17:39:11 lbernardo Exp $
+ *  $Id: platform.h,v 1.2 2009-11-24 23:53:40 sampo Exp $
  */
 
 #ifndef _platform_h
@@ -10,6 +10,8 @@
 #define BADFD (INVALID_HANDLE_VALUE)
 #define closefile(x) (CloseHandle(x)?0:-1)
 #define openfile_ro(path) CreateFile((path), GENERIC_READ, FILE_SHARE_READ, 0 /*security*/, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0)
+#define geteuid() 0
+#define getegid() 0
 #else
 #define fdtype int
 #define BADFD (-1)
