@@ -265,7 +265,7 @@ int zxid_az_cf_ses(struct zxid_conf* cf, const char* qs, struct zxid_ses* ses)
   zxid_parse_cgi(&cgi, "");
   DD("qs(%s) ses=%p", STRNULLCHKD(qs), ses);
   if (qs && ses)
-    zxid_add_qs_to_pool(cf, ses, zx_dup_cstr(cf->ctx, qs), 1);
+    zxid_add_qs_to_ses(cf, ses, zx_dup_cstr(cf->ctx, qs), 1);
   ret =  zxid_pep_az_soap(cf, &cgi, ses, cf->pdp_call_url?cf->pdp_call_url:cf->pdp_url);
   D_DEDENT("az: ");
   return ret;
