@@ -662,6 +662,17 @@ struct zx_str* zxid_call(struct zxid_conf* cf, struct zxid_ses* ses, const char*
   struct zx_e_Envelope_s* env;
   struct zx_a_EndpointReference_s* epr;
   D_INDENT("call: ");
+
+  if (!cf || !ses || !enve) {
+    ERR("Missing mandatory arguments ses=%p", ses);
+    return 0;
+  }
+
+  if (!memcmp(enve, "<?xml ", sizeof("<?xml ")-1)) {  /* Ignore common, but unnecessary decl. */
+    for (enve += sizeof("<?xml "); *enve && !(enve[0] == '?' && enve[1] == '>'); ++enve) ;
+    if (*enve)
+      enve += 2;
+  }
   
   if (memcmp(enve, "<e:Envelope", sizeof("<e:Envelope")-1)) {
     if (memcmp(enve, "<e:Body", sizeof("<e:Body")-1)) {
